# -*- coding: utf-8 -*-
from Products.CMFCore import utils as cmf_utils
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.permissions import AddPortalFolders
from Products.CMFDynamicViewFTI import content_for_tests
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation


def initialize(context):
    # (DynamicViewTypeInformation factory is created from ZCML)
    cmf_utils.registerIcon(
        DynamicViewTypeInformation,
        'images/typeinfo.gif',
        globals()
    )

    context.registerClass(
        content_for_tests.DynFolder,
        permission=AddPortalFolders,
        constructors=(
            ('addDynFolder', content_for_tests.addDynFolder),
        ),
        icon='images/typeinfo.gif'
    )

    context.registerClass(
        content_for_tests.DynDocument,
        permission=AddPortalContent,
        constructors=(
            ('addDynDocument', content_for_tests.addDynDocument),
        ),
        icon='images/typeinfo.gif'
    )



# -*- coding: utf-8 -*-
#
# CMFDVFTITestCase
#
from plone.app import testing
from plone.app.testing import bbb
from Products.CMFCore.interfaces import ISiteRoot
from Products.GenericSetup import EXTENSION, profile_registry


class PloneTestCaseFixture(bbb.PloneTestCaseFixture):

    defaultBases = (bbb.PTC_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        profile_registry.registerProfile(
            'CMFDVFTI_sampletypes',
            'CMFDynamicViewFTI Sample Content Types',
            'Extension profile including CMFDVFTI sample content types',
            'profiles/sample_types',
            'CMFDynamicViewFTI',
            EXTENSION,
            for_=ISiteRoot
        )
        import Products.CMFDynamicViewFTI.tests
        self.loadZCML(name='browserdefault.zcml',
                      package=Products.CMFDynamicViewFTI.tests)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'CMFDynamicViewFTI:CMFDVFTI_sampletypes')

    def tearDownZope(self, app):
        pass

CDV_FIXTURE = PloneTestCaseFixture()
CDV_FUNCTIONAL_TESTING = testing.FunctionalTesting(
    bases=(CDV_FIXTURE, ), name='CMFDynamicViewFTI Testing:Functional')


class CMFDVFTITestCase(bbb.PloneTestCase):
    """This is a stub now, but in case you want to try
       something fancy on Your Branch (tm), put it here.
    """

    layer = CDV_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from Products.Archetypes.atapi import StringField
from Products.CMFCore.interfaces import ITypeInformation
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from Products.CMFDynamicViewFTI.tests import CMFDVFTITestCase
from zope.interface.verify import verifyObject
import transaction

fti_meta_type = DynamicViewTypeInformation.meta_type


class TestFTI(CMFDVFTITestCase.CMFDVFTITestCase):

    def afterSetUp(self):
        self.types = getToolByName(self.portal, 'portal_types')
        self.fti = self.types['DynFolder']

    def _makeOne(self):
        # Create and return a DynFolder
        self.folder.invokeFactory('DynFolder', id='dynfolder')
        return self.folder.dynfolder

    def test_doesImplementITypeInformation(self):
        iface = ITypeInformation
        self.failUnless(iface.providedBy(self.fti))
        self.failUnless(verifyObject(iface, self.fti))

    def test_doesImplementIDynamicViewTypeInformation(self):
        iface = IDynamicViewTypeInformation
        self.failUnless(iface.providedBy(self.fti))
        self.failUnless(verifyObject(iface, self.fti))

    def test_meta_type(self):
        self.failUnlessEqual(self.fti.meta_type, fti_meta_type)

    def test_paranoid_subclass_test(self):
        self.failUnless(isinstance(self.fti, DynamicViewTypeInformation))

    def test_CreateDynFolder(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getPortalTypeName(), 'DynFolder')
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getId(), 'DynFolder')
        self.assertEqual(info.Title(), 'DynFolder')
        self.assertEqual(info.getDefaultViewMethod(dynfolder), 'index_html')
        self.assertEqual(
            info.getAvailableViewMethods(dynfolder),
            ('index_html', 'custom_view', 'zope3_view')
        )

    def test_DynFolderDefaultView(self):
        dynfolder = self._makeOne()
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'index_html')

    def test_DynFolderCustomView(self):
        dynfolder = self._makeOne()
        self.types.DynFolder.manage_changeProperties(
            default_view='custom_view'
        )
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'custom_view')

    def test_DynFolderViewFromLayout(self):
        dynfolder = self._makeOne()
        dynfolder.layout = 'custom_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'custom_view')

    def test_DynFolderViewFromCallableLayout(self):
        dynfolder = self._makeOne()
        dynfolder.layout = lambda: 'custom_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'custom_view')

    def test_NoneLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = None
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'index_html')

    def test_EmptyLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = ''
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getViewMethod(dynfolder), 'index_html')

    def test_InvalidLayoutRaisesTypeError(self):
        dynfolder = self._makeOne()
        dynfolder.layout = object()
        info = self.types.getTypeInfo(dynfolder)
        self.assertRaises(TypeError, info.getViewMethod, dynfolder)

    def test_EnforceLayoutAvailable(self):
        dynfolder = self._makeOne()
        dynfolder.layout = 'custom_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, enforce_available=True),
            'custom_view'
        )

    def test_UnavailableLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = 'bad_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, enforce_available=True),
            'index_html'
        )

    def test_CheckLayoutExists(self):
        dynfolder = self._makeOne()
        dynfolder.manage_addDTMLMethod('custom_view', file='')
        dynfolder.layout = 'custom_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, check_exists=True),
            'custom_view'
        )

    def test_MissingLayoutReturnsDefaultView(self):
        dynfolder = self._makeOne()
        dynfolder.layout = 'bad_view'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getViewMethod(dynfolder, check_exists=True),
            'index_html'
        )

    def test_DynFolderDefaultPage(self):
        dynfolder = self._makeOne()
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_DynFolderDefaultPageFromAttribute(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = 'custom_page'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), 'custom_page')

    def test_DynFolderDefaultPageFromCallable(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = lambda: 'custom_page'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), 'custom_page')

    def test_NoneDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = None
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_EmptyDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = ''
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)

    def test_InvalidDefaultPageRaisesTypeError(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = object()
        info = self.types.getTypeInfo(dynfolder)
        self.assertRaises(TypeError, info.getDefaultPage, dynfolder)

    def test_CheckDefaultPageExists(self):
        dynfolder = self._makeOne()
        dynfolder.manage_addDTMLMethod('custom_page', file='')
        dynfolder.default_page = 'custom_page'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getDefaultPage(dynfolder, check_exists=True),
            'custom_page'
        )

    def test_MissingDefaultPageReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.default_page = 'bad_page'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(
            info.getDefaultPage(dynfolder, check_exists=True),
            None
        )

    def test_NonFolderishObjectReturnsNone(self):
        dynfolder = self._makeOne()
        dynfolder.isPrincipiaFolderish = 0
        dynfolder.default_page = 'custom_page'
        info = self.types.getTypeInfo(dynfolder)
        self.assertEqual(info.getDefaultPage(dynfolder), None)


class TestEmptyLayoutBug(CMFDVFTITestCase.CMFDVFTITestCase):
    # Finally, here is why we did all this...

    def afterSetUp(self):
        # Make a DynFolder
        self.folder.invokeFactory('DynFolder', id='dynfolder')
        self.dynfolder = self.folder.dynfolder
        self.dynfolder.layout = ''  # Empty layout triggers bug
        self.dynfolder_path = self.dynfolder.absolute_url(1)

        # Make a DynDocument
        self.folder.invokeFactory('DynDocument', id='dyndocument')
        self.dyndocument = self.folder.dyndocument
        self.dyndocument.layout = ''  # Empty layout triggers bug
        self.dyndocument_path = self.dyndocument.absolute_url(1)

        self.basic = '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD)

    def test_FolderEmptyLayoutBug(self):
        response = self.publish(
            self.dynfolder_path + '/view',
            basic=self.basic
        )
        self.assertEqual(response.getStatus(), 200)

    def test_DocumentEmptyLayoutBug(self):
        # add a text field to dyndocument which is not present
        # but needed for the standard view
        self.dyndocument.Schema().addField(StringField('text'))
        response = self.publish(
            self.dyndocument_path + '/view',
            basic=self.basic
        )
        self.assertEqual(response.getStatus(), 200)


class TestModifyDefaultPage(CMFDVFTITestCase.CMFDVFTITestCase):

    def _makeOne(self):
        # Create and return a DynFolder
        self.folder.invokeFactory('DynFolder', id='dynfolder')
        dynfolder = self.folder.dynfolder
        dynfolder.invokeFactory('DynDocument', id='default_document')
        dynfolder.setDefaultPage('default_document')
        return dynfolder

    def test_rename_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), 'default_document')
        transaction.commit()
        dynfolder.manage_renameObject('default_document', 'renamed_default')
        self.assertFalse('default_document' in dynfolder.objectIds())
        self.assertTrue('renamed_default' in dynfolder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), 'renamed_default')
        self.assertEqual(dynfolder.getProperty('default_page', ''),
                         'renamed_default')

    def test_delete_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), 'default_document')
        dynfolder.manage_delObjects(['default_document'])
        self.assertFalse('default_document' in dynfolder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), None)
        self.assertEqual(dynfolder.getProperty('default_page', ''), '')

    def test_cut_default_page(self):
        dynfolder = self._makeOne()
        self.assertEqual(dynfolder.getDefaultPage(), 'default_document')
        transaction.commit()
        clipboard = dynfolder.manage_cutObjects(['default_document'])
        self.folder.manage_pasteObjects(clipboard)
        self.assertFalse('default_document' in dynfolder.objectIds())
        self.assertTrue('default_document' in self.folder.objectIds())
        self.assertEqual(dynfolder.getDefaultPage(), None)
        self.assertEqual(dynfolder.getProperty('default_page', ''), '')



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from App.class_init import InitializeClass
from Products.CMFCore.permissions import View
from Products.CMFCore.TypesTool import FactoryTypeInformation
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from types import ClassType
from zope.interface import implementer


def safe_hasattr(obj, name, _marker=object()):
    """Make sure we don't mask exceptions like hasattr().

    We don't want exceptions other than AttributeError to be masked,
    since that too often masks other programming errors.
    Three-argument getattr() doesn't mask those, so we use that to
    implement our own hasattr() replacement.
    """
    return getattr(obj, name, _marker) is not _marker


def safe_callable(obj):
    """Make sure our callable checks are ConflictError safe."""
    if safe_hasattr(obj, '__class__'):
        if safe_hasattr(obj, '__call__'):
            return True
        else:
            return isinstance(obj, ClassType)
    else:
        return callable(obj)


def om_has_key(context, key):
    """Object Manager has_key method with optimization for btree folders

    Zope's OFS.ObjectManager has no method for checking if an object with an id
    exists inside a folder.
    """
    klass = getattr(aq_base(context), '__class__', None)
    if hasattr(klass, 'has_key'):
        # BTreeFolder2 optimization
        if key in context:
            return True
    else:
        # standard ObjectManager api
        if key in context.objectIds():
            return True
    return False

fti_meta_type = 'Factory-based Type Information with dynamic views'


@implementer(IDynamicViewTypeInformation)
class DynamicViewTypeInformation(FactoryTypeInformation):
    """FTI with dynamic views

    A value of (dynamic view) as alias is replaced by the output of
    defaultView()
    """

    meta_type = fti_meta_type
    security = ClassSecurityInfo()

    _properties = FactoryTypeInformation._properties + (
        {
            'id': 'default_view',
            'type': 'string',
            'mode': 'w',
            'label': 'Default view method',
        },
        {
            'id': 'view_methods',
            'type': 'lines',
            'mode': 'w',
            'label': 'Available view methods'
        },
        {
            'id': 'default_view_fallback',
            'type': 'boolean',
            'mode': 'w',
            'label': 'Fall back to default view?'
        },
    )

    default_view = ''
    view_methods = ()
    default_view_fallback = False

    def manage_changeProperties(self, **kw):
        """Overwrite change properties to verify that default_view is in the method
        list
        """
        FactoryTypeInformation.manage_changeProperties(self, **kw)
        default_view = self.default_view
        view_methods = self.view_methods
        if not default_view:
            # TODO: use view action
            self.default_view = default_view = self.immediate_view
        if not view_methods:
            self.view_methods = view_methods = (default_view, )
        if default_view and default_view not in view_methods:
            raise ValueError("%s not in %s" % (default_view, view_methods))

    @security.protected(View)
    def getDefaultViewMethod(self, context):
        # Get the default view method from the FTI.
        return str(self.default_view)

    @security.protected(View)
    def getAvailableViewMethods(self, context):
        # Get a list of registered view methods.
        methods = self.view_methods
        if isinstance(methods, basestring):
            methods = (methods, )
        return tuple(methods)

    @security.protected(View)
    def getViewMethod(
        self,
        context,
        enforce_available=False,
        check_exists=False
    ):
        # Get view method (aka layout) name from context.
        # Return -- view method from context or default view name.
        default = self.getDefaultViewMethod(context)
        layout = getattr(aq_base(context), 'layout', None)

        if safe_callable(layout):
            layout = layout()
        if not layout:
            return default
        if not isinstance(layout, basestring):
            raise TypeError(
                "layout of %s must be a string, got %s" %
                (repr(context), type(layout))
            )
        if enforce_available:
            available = self.getAvailableViewMethods(context)
            if layout not in available:
                return default
        if check_exists:
            method = getattr(context, layout, None)
            if method is None:
                return default
        return layout

    @security.protected(View)
    def getDefaultPage(self, context, check_exists=False):
        # Get the default page from a folderish object.
        #
        # Non folderish objects don't have a default view.
        #
        # If check_exists is enabled the method makes sure the object with the
        # default page id exists.
        #
        # Return -- None for no default page or a string
        if not getattr(aq_base(context), 'isPrincipiaFolderish', False):
            # non folderish objects don't have a default page per se
            return None

        default_page = getattr(aq_base(context), 'default_page', None)

        if safe_callable(default_page):
            default_page = default_page()
        if not default_page:
            return None
        if isinstance(default_page, (tuple, list)):
            default_page = default_page[0]
        if not isinstance(default_page, str):
            raise TypeError(
                "default_page must be a string, got %s(%s):" %
                (default_page, type(default_page))
            )

        if check_exists and not om_has_key(context, default_page):
            return None

        return default_page

    @security.protected(View)
    def defaultView(self, context):
        # Get the current view to use for an object. If a default page is set,
        # use that, else use the currently selected view method/layout.

        # Delegate to PloneTool's version if we have it else, use own rules
        plone_utils = getToolByName(self, 'plone_utils', None)
        if plone_utils is not None:
            obj, path = plone_utils.browserDefault(context)
            return path[-1]
        else:
            default_page = self.getDefaultPage(context, check_exists=True)
            if default_page is not None:
                return default_page
            fallback = self.default_view_fallback
            return self.getViewMethod(context, check_exists=fallback)

    security.declarePublic('queryMethodID')

    def queryMethodID(self, alias, default=None, context=None):
        # Query method ID by alias.

        # Use "(dynamic view)" as the alias target to look up as per
        # defaultView()

        # Use "(selected layout)" as the alias target to look up as per
        # getViewMethod()

        methodTarget = FactoryTypeInformation.queryMethodID(
            self,
            alias,
            default=default,
            context=context
        )
        if not isinstance(methodTarget, basestring):
            # nothing to do, method_id is probably None
            return methodTarget

        if context is None or default == '':
            # the edit zpts like typesAliases don't apply a context and set the
            # default to ''. We do not want to resolve (dynamic view) for these
            # methods.
            return methodTarget

        # Our two special targets:

        if methodTarget.lower() == "(dynamic view)":
            methodTarget = self.defaultView(context)

        if methodTarget.lower() == "(selected layout)":
            fallback = self.default_view_fallback
            methodTarget = self.getViewMethod(context, check_exists=fallback)

        return methodTarget

InitializeClass(DynamicViewTypeInformation)



# -*- coding: utf-8 -*-
# BBB module will be removed at some point. We will still need it as long as
# we plan to support Plone 4.3.x
from Products.CMFDynamicViewFTI.interfaces import *
import warnings

warnings.warn(
    'import from Products.CMFDynamicViewFTI.interfaces instead',
    DeprecationWarning
)



# -*- coding: utf-8 -*-
"""
Add DynamicView FTI form (ZMI)
"""
# $Id$

from Products.CMFCore.browser.typeinfo import FactoryTypeInformationAddView
from Products.CMFDynamicViewFTI import DynamicViewTypeInformation


class DVFactoryTypeInformationAddView(FactoryTypeInformationAddView):
    """See FactoryTypeInformationAddView that does all the job"""

    klass = DynamicViewTypeInformation

    description = u'A dynamic view type information object defines a portal type.'



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.CMFDynamicViewFTI.interfaces import IBrowserDefault
from Products.CMFDynamicViewFTI.interfaces import ISelectableBrowserDefault
from Products.CMFDynamicViewFTI.tests import CMFDVFTITestCase
from zope.interface import directlyProvides
from zope.interface import Interface
from zope.interface.verify import verifyClass
from zope.publisher.browser import TestRequest


class DummyFolder(BrowserDefaultMixin):

    def getTypeInfo(self):
        return self.fti


class IDummy(Interface):
    """ marker interface for a zope 3 view """


class TestBrowserDefault(CMFDVFTITestCase.CMFDVFTITestCase):

    def test_doesImplementISelectableBrowserDefault(self):
        iface = ISelectableBrowserDefault
        self.failUnless(iface.implementedBy(BrowserDefaultMixin))
        self.failUnless(verifyClass(iface, BrowserDefaultMixin))

    def test_extendsInterface(self):
        self.failUnless(ISelectableBrowserDefault.extends(IBrowserDefault))


class TestAvailableLayouts(CMFDVFTITestCase.CMFDVFTITestCase):

    def afterSetUp(self):
        self.types = getToolByName(self.portal, 'portal_types')

        self.dfolder = DummyFolder()
        self.dfolder.fti = self.types['DynFolder']

    def test_Zope3View(self):
        dfolder = self.dfolder
        dfolder.layout = 'zope3_view'
        dfolder.REQUEST = TestRequest()
        view_methods = dfolder.getAvailableLayouts()
        view_ids = [view_id for view_id, foo in view_methods]
        self.failIf(dfolder.layout in view_ids)

        # Mark the object with interface connected to the zope 3 view
        directlyProvides(dfolder, IDummy)
        view_methods = dfolder.getAvailableLayouts()
        view_ids = [view_id for view_id, foo in view_methods]
        self.failIf(dfolder.layout not in view_ids)

    def test_Zope3ViewTitle(self):
        dfolder = self.dfolder
        dfolder.layout = 'zope3_view'
        dfolder.REQUEST = TestRequest()
        directlyProvides(dfolder, IDummy)
        view_methods = dfolder.getAvailableLayouts()

        for id, title in view_methods:
            if id == dfolder.layout:
                self.assertEqual(title, 'Zope3 Test View')



# -*- coding: utf-8 -*-
"""Mixin class for selectable views

This module contains a mixin-class to support selecting default layout
templates and/or default pages (in the style of default_page/index_html).
The implementation extends TemplateMixin from Archetypes, and implements
the ISelectableBrowserDefault interface from CMFPlone.
"""
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from App.class_init import InitializeClass
from ExtensionClass import Base
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation
from Products.CMFDynamicViewFTI.interfaces import ISelectableBrowserDefault
from Products.CMFDynamicViewFTI.permissions import ModifyViewTemplate
from zope.browsermenu.interfaces import IBrowserMenu
from zope.component import getSiteManager, getUtility
from zope.interface import Interface, implementer, providedBy

_marker = object()
fti_meta_type = DynamicViewTypeInformation.meta_type


@implementer(ISelectableBrowserDefault)
class BrowserDefaultMixin(Base):
    """Mixin class for content types using the dynamic view FTI

    Allow the user to select a layout template (in the same way as
    TemplateMixin in Archetypes does), and/or to set a contained
    object's id as a default_page (acting in the same way as index_html)

    Note: folderish content types should overwrite HEAD like ATContentTypes
    """

    _at_fti_meta_type = fti_meta_type
    aliases = {
        '(Default)': '(dynamic view)',
        'view': '(selected layout)',
        'edit': 'base_edit',
        'properties': 'base_metadata',
        'sharing': 'folder_localrole_form',
        'gethtml': '',
        'mkdir': '',
        }

    default_view = "base_view"
    suppl_views = ()

    security = ClassSecurityInfo()

    @security.protected(View)
    def defaultView(self, request=None):
        # Get the actual view to use. If a default page is set, its id will
        # be returned. Else, the current layout's page template id is returned.
        fti = self.getTypeInfo()
        if fti is None:
            return self.default_view
        else:
            return fti.defaultView(self)

    @security.protected(View)
    def __call__(self):
        """
        Resolve and return the selected view template applied to the object.
        This should not consider the default page.
        """
        template = self.unrestrictedTraverse(self.getLayout())
        return template()

    @security.protected(View)
    def getDefaultPage(self):
        # Return the id of the default page, or None if none is set.
        # The default page must be contained within this (folderish) item.
        fti = self.getTypeInfo()
        if fti is None:
            return None

        plone_utils = getToolByName(self, 'plone_utils', None)
        if plone_utils is not None:
            return plone_utils.getDefaultPage(self)

        return fti.getDefaultPage(self, check_exists=True)

    @security.protected(View)
    def getLayout(self, **kw):
        # Get the selected view method.
        # Note that a selected default page will override the view method.
        fti = self.getTypeInfo()
        if fti is None:
            return None
        return fti.getViewMethod(self)

    @security.public
    def canSetDefaultPage(self):
        # Check if the user has permission to select a default page on this
        # (folderish) item, and the item is folderish.
        if not self.isPrincipiaFolderish:
            return False
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        return member.has_permission(ModifyViewTemplate, self)

    @security.protected(ModifyViewTemplate)
    def setDefaultPage(self, objectId):
        # Set the default page to display in this (folderish) object.

        # The objectId must be a value found in self.objectIds() (i.e. a
        # contained object). This object will be displayed as the
        # default_page/index_html object of this (folderish) object. This will
        # override the current layout template returned by getLayout().
        # Pass None for objectId to turn off the default page and return to
        # using the selected layout template.
        new_page = old_page = None
        if objectId is not None:
            new_page = getattr(self, objectId, None)
        if self.hasProperty('default_page'):
            pages = self.getProperty('default_page', '')
            if isinstance(pages, (list, tuple)):
                for page in pages:
                    old_page = getattr(self, page, None)
                    if page is not None:
                        break
            elif isinstance(pages, str):
                old_page = getattr(self, pages, None)

            if objectId is None:
                self.manage_delProperties(['default_page'])
            else:
                self.manage_changeProperties(default_page=objectId)
        else:
            if objectId is not None:
                self.manage_addProperty('default_page', objectId, 'string')
        if new_page != old_page:
            if new_page is not None:
                new_page.reindexObject(['is_default_page'])
            if old_page is not None:
                old_page.reindexObject(['is_default_page'])

    @security.protected(ModifyViewTemplate)
    def setLayout(self, layout):
        # Set the layout as the current view.

        # 'layout' should be one of the list returned by getAvailableLayouts(),
        # but it is not enforced. If a default page has been set with
        # setDefaultPage(), it is turned off by calling setDefaultPage(None).
        if not (layout and isinstance(layout, basestring)):
            raise ValueError(
                "layout must be a non empty string, got %s(%s)" %
                (layout, type(layout))
            )

        defaultPage = self.getDefaultPage()
        if defaultPage is None and layout == self.getLayout():
            return

        if self.hasProperty('layout'):
            self.manage_changeProperties(layout=layout)
        else:
            if getattr(aq_base(self), 'layout', _marker) is not _marker:
                # Archetypes remains? clean up
                old = self.layout
                if old and not isinstance(old, basestring):
                    raise RuntimeError(
                        "layout attribute exists on %s and is no string: %s" %
                        (self, type(old))
                    )
                delattr(self, 'layout')

            self.manage_addProperty('layout', layout, 'string')

        self.setDefaultPage(None)

    @security.protected(View)
    def getDefaultLayout(self):
        # Get the default layout method.
        fti = self.getTypeInfo()
        if fti is None:
            return "base_view"  # XXX
        return fti.getDefaultViewMethod(self)

    @security.public
    def canSetLayout(self):
        # Check if the current authenticated user is permitted to select a layout.
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        return member.has_permission(ModifyViewTemplate, self)

    @security.protected(View)
    def getAvailableLayouts(self):
        # Get the layouts registered for this object from its FTI.
        fti = self.getTypeInfo()
        if fti is None:
            return ()
        result = []
        method_ids = fti.getAvailableViewMethods(self)
        spec = (providedBy(self), providedBy(self.REQUEST))
        gsm = getSiteManager()
        for mid in method_ids:
            factory = gsm.adapters.lookup(spec, Interface, mid)
            if factory is not None:
                menu = getUtility(
                    IBrowserMenu,
                    'plone_displayviews'
                )
                item = menu.getMenuItemByAction(self, self.REQUEST, mid)
                title = item and item.title or mid
                result.append((mid, title))
            else:
                method = getattr(self, mid, None)
                if method is not None:
                    # a method might be a template, script or method
                    try:
                        title = method.aq_inner.aq_explicit.title_or_id()
                    except AttributeError:
                        title = mid
                    result.append((mid, title))
        return result

InitializeClass(BrowserDefaultMixin)


def check_default_page(obj, event):
    """event subscriber, unset default page if target no longer exists

    used by default for zope.container.interfaces.IContainerModifiedEvent
    """
    container = obj
    default_page_id = container.getProperty('default_page', '')
    if default_page_id and not (default_page_id in container.objectIds()):
        ISelectableBrowserDefault(container).setDefaultPage(None)


def rename_default_page(obj, event):
    """event subscriber, rename default page if targte was renamed

    used by default for zope.lifecycleevent.interfaces.IObjectMovedEvent
    """
    newParent = event.newParent
    if newParent != event.oldParent:
        return
    elif ISelectableBrowserDefault.providedBy(newParent):
        default_page_id = newParent.getProperty('default_page', '')
        if default_page_id == event.oldName:
            ISelectableBrowserDefault(newParent).setDefaultPage(event.newName)



# -*- coding: utf-8 -*-
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class DynFolder(PortalFolder, BrowserDefaultMixin):
    pass


class DynDocument(PortalContent, BrowserDefaultMixin):

    def __init__(self, id, title):
        self.id = id
        self.title = title


def addDynFolder(self, id, title='', REQUEST=None):
    """Add a new DynFolder object with id *id*.
    """
    ob = DynFolder(id, title)
    self._setObject(id, ob, suppress_events=True)


def addDynDocument(self, id, title='', REQUEST=None):
    """Add a new DynDocument object with id *id*.
    """
    ob = DynDocument(id, title)
    self._setObject(id, ob, suppress_events=True)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '4.1.5'

long_description = open("README.rst").read()
long_description += "\n"
long_description += open("CHANGES.rst").read()


setup(
    name='Products.CMFDynamicViewFTI',
    version=version,
    description="CMFDynamicViewFTI is a product for dynamic views in CMF.",
    long_description=long_description,
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='Zope CMF Plone dynamic view',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.python.org/pypi/Products.CMFDynamicViewFTI',
    license='ZPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['Products'],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            'plone.app.contentmenu',
            'zope.publisher',
            'plone.app.testing >= 4.2.5',
            'Products.Archetypes',
        ]
    ),
    install_requires=[
        'setuptools',
        'zope.browsermenu',
        'zope.component',
        'zope.interface',
        'Products.CMFCore',
        'Products.GenericSetup',
        'Acquisition',
        'ExtensionClass',
        'Zope2',
    ],
)


# -*- coding: utf-8 -*-
from Products.CMFCore.interfaces import ITypeInformation
from zope.interface import Interface


class IDynamicViewTypeInformation(ITypeInformation):
    """Interface for FTI with dynamic views

    A value of (dynamic view) as alias is replaced by the output of getLayout()
    """

    def getAvailableViewMethods(context):
        """Get a list of registered view methods
        """

    def getViewMethod(context, enforce_available=True):
        """Get view method name from context

        Return -- view method from context or default view name
        """

    def getDefaultViewMethod(context):
        """Get the default view method from the FTI
        """

    def getDefaultPage(context, check_exists=False):
        """Get the default page from a folderish object

        Non folderish objects don't have a default view.

        If check_exists is enabled the method makes sure the object with the
        default page id exists.

        Return -- None for no default page or a string
        """

    def defaultView(context):
        """Get the layout for an object

        At first it tries to get the default page from the context.
        A default page must be listed on the folder or else it is ignored.

        At last it get the view method.

        Return -- a string containing the name of the layout
        """


class IBrowserDefault(Interface):
    """Content supporting different views on a per-instance basis.

    This can be either as a page template (a layout), or as the id of a
    contained object (aka a default page, set inside a folderish item only).
    """

    def defaultView(request=None):
        """Get the actual view to use.

        If a default page is set, its id will
        be returned. Else, the current layout's page template id is returned.
        """

    def __call__():
        """Resolve and return the selected view template applied to the object.

        This should not consider any default page set.
        """

    def getDefaultPage():
        """Return the id of the default page, or None if none is set.

        The default page must be contained within this (folderish) item.
        """

    def getLayout(**kw):
        """Get the selected layout template.

        Note that a selected default page will override the layout template.
        """

    def getDefaultLayout():
        """Get the default layout template.
        """


class ISelectableBrowserDefault(IBrowserDefault):
    """Content supporting operations to explicitly set the default layout
    template or default page object.
    """

    def canSetDefaultPage():
        """Can a default page be set?

        Return True if the user has permission to select a default page on this
        (folderish) item, and the item is folderish.
        """

    def setDefaultPage(objectId):
        """Set the default page to display in this (folderish) object.

        The objectId must be a value found in self.objectIds() (i.e. a
        contained object). This object will be displayed as the
        default_page/index_html object of this (folderish) object. This will
        override the current layout template returned by getLayout(). Pass None
        for objectId to turn off the default page and return to using the
        selected layout template.
        """

    def canSetLayout():
        """Return True if the current authenticated user is permitted to select
        a layout.
        """

    def setLayout(layout):
        """Set the layout as the current view.

        'layout' should be one of the list returned by getAvailableLayouts().
        If a default page has been set with setDefaultPage(), it is turned off
        by calling setDefaultPage(None).
        """

    def getAvailableLayouts():
        """Get the layouts registered for this object.

        This should return a list of tuples: (id, title), where id is the id
        of the page template providing the layout and title is the title of
        that page template as it will be displayed to the user.
        """
